/**
 * @file LCD_mini_click.c
 * @brief <h2> LCD mini click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * LCD mini click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page Example
 * @date        14 Dec 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    FT900Q
 * -<b> Dev. Board    </b> :    EasyFT90x v7
 * -<b> Ext. Modules  </b> :    LCD mini click
 * -<b> SW            </b> :    mikroC PRO for FT90x v.2.0.0
 *
 */

/**
 * @page Dev. board setup
 *
 * 1) Put LCD mini click into mikroBUS 1 slot.
 *
 */


/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>

/******************************************************************************
* Preprocessors
*******************************************************************************/


/******************************************************************************
* Typedefs
*******************************************************************************/


/******************************************************************************
* Variable Definitions
*******************************************************************************/
sbit DigPot_CS 	at GPIO_PIN9_bit;

sbit SPExpanderRST 	at GPIO_PIN1_bit;
sbit SPExpanderCS 	at GPIO_PIN28_bit;

static uint8_t value = 0x00;

static unsigned int pwm_period;

/******************************************************************************
* Function Prototypes
*******************************************************************************/

static void system_init( void );
static void set_contrast( uint8_t value );
static void set_bcklight( uint8_t value );

/******************************************************************************
* Private Functions
*******************************************************************************/

/**************************************************************************//**
 *
 */
static void system_init( void )
{
    DigPot_CS = 1;
    GPIO_Digital_Output( &GPIO_PORT_08_15, _GPIO_PINMASK_1 );

    // Initialise SPI.
    SPIM1_Init_Advanced( 
        _SPI_MASTER_CLK_RATIO_64, 
        _SPI_CFG_PHASE_CAPTURE_RISING | 
        _SPI_CFG_POLARITY_IDLE_LOW | 
        _SPI_CFG_SS_AUTO_DISABLE | 
        _SPI_CFG_FIFO_DISABLE, 
        _SPI_SS_LINE_NONE
    );

    // Initialise PWM for controlling back light.                                   
    pwm_period = PWM_Init(5000);
    PWM_ConfigChannel(0, _PWM_INIT_STATE_0, _PWM_CHANNEL_0);
    PWM_EnableOutput();

    SPI_Lcd_Config(0);                     // Initialize Lcd over SPI interface
    SPI_Lcd_Cmd(_LCD_CLEAR);               // Clear display
    SPI_Lcd_Cmd(_LCD_CURSOR_OFF);          // Turn cursor off
}

/**************************************************************************//**
 *
 */
static void set_contrast( uint8_t value )
{
    DigPot_CS = 0;

    SPIM1_Write(0x00);
    SPIM1_Write(~value);

    DigPot_CS = 1;
}

/**************************************************************************//**
 *
 */
static void set_bcklight( uint8_t value )
{
    PWM_ConfigChannel((uint16_t) floor(~value / 255.0f * pwm_period),
                                 _PWM_INIT_STATE_0, _PWM_CHANNEL_0);
}

/******************************************************************************
* Public Functions
*******************************************************************************/

void main()
{
    system_init();
    
    SPI_Lcd_Out(1, 6, "mikroE");
    SPI_Lcd_Out(2, 2, "LCD mini click");

    set_bcklight(0xFF);
    set_contrast(0xDF);
    
    Delay_ms(5000);

    while (1)
    {
        set_bcklight(value);
        set_contrast(value);
        
        value++;
        delay_ms(40);
    }
}

/*************** END OF FUNCTIONS *********************************************/